/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */

/* This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * (C) Copyright 2009 Novell, Inc.
 */

#include <string.h>
#include <glib/gi18n.h>
#include "nmn-item-renderer.h"
#include "nm-list-item.h"
#include "nm-icon-cache.h"
#include "nm-connection-item.h"
#include "nm-device-item.h"
#include "nmn-connection-details.h"

G_DEFINE_TYPE (NmnItemRenderer, nmn_item_renderer, GTK_TYPE_EVENT_BOX)

enum {
    PROP_0,
    PROP_ITEM,

    LAST_PROP
};

#define GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), NMN_TYPE_ITEM_RENDERER, NmnItemRendererPrivate))

typedef struct {
    NMListItem *item;

    GtkBox *vbox; /* child of self */
    GtkBox *hbox; /* child of vbox */

    GtkImage *icon;
    GtkLabel *name_and_status;
    GtkLabel *security_label;
    GtkWidget *expander;
    NmnConnectionDetails *details;
    GtkWidget *connect_button;
    GtkWidget *remove_button;

    gboolean prelight;
    GdkColor prelight_color;
    GdkColor connection_item_color;

    gulong item_changed_id;

    gboolean disposed;
} NmnItemRendererPrivate;

GtkWidget *
nmn_item_renderer_new (void)
{
    return (GtkWidget *) g_object_new (NMN_TYPE_ITEM_RENDERER, NULL);
}

NMListItem *
nmn_item_renderer_get_item (NmnItemRenderer *self)
{
    g_return_val_if_fail (NMN_IS_ITEM_RENDERER (self), NULL);

    return GET_PRIVATE (self)->item;
}

static void
update_background (NmnItemRenderer *self)
{
    NmnItemRendererPrivate *priv = GET_PRIVATE (self);
    GdkColor *color;

    if (priv->prelight) {
        color = &priv->prelight_color;
    } else {
        if (NM_IS_CONNECTION_ITEM (priv->item) && nm_connection_item_get_connection (NM_CONNECTION_ITEM (priv->item)))
            color = &priv->connection_item_color;
        else
            color = NULL;
    }

    gtk_widget_modify_bg (GTK_WIDGET (self), GTK_STATE_NORMAL, color);
}

static void
update_details (NmnItemRenderer *self)
{
    NmnItemRendererPrivate *priv = GET_PRIVATE (self);
    NMConnection *connection;
    NMDevice *device;
    NMSettingIP4Config *setting;
    NMIP4Config *config;

    if (!priv->details)
        return;

    connection = (NMConnection *) nm_connection_item_get_connection (NM_CONNECTION_ITEM (priv->item));
    if (!connection)
        return;

    device = nm_device_item_get_device (NM_DEVICE_ITEM (priv->item));
    setting = (NMSettingIP4Config *) nm_connection_get_setting (connection, NM_TYPE_SETTING_IP4_CONFIG);
    
    if (device && nm_device_get_state (device) == NM_DEVICE_STATE_ACTIVATED)
        config = nm_device_get_ip4_config (device);
    else
        config = NULL;

    nmn_connection_details_set_data (priv->details, setting, config);
}

static void
item_changed (NMListItem *item,
              GParamSpec *spec,
              gpointer user_data)
{
    NmnItemRenderer *self = NMN_ITEM_RENDERER (user_data);
    NmnItemRendererPrivate *priv = GET_PRIVATE (self);
    const char *property = spec ? spec->name : NULL;
    char *str;

    if (!property || !strcmp (property, NM_LIST_ITEM_NAME) || !strcmp (property, NM_LIST_ITEM_STATUS)) {
        const char *status_str;
        const char *button_label;
        NMListItemStatus status;

        status = nm_list_item_get_status (priv->item);
        switch (status) {
        case NM_LIST_ITEM_STATUS_CONNECTED:
            status_str = _("Connected");
            button_label = _("Disconnect");
            break;
        case NM_LIST_ITEM_STATUS_CONNECTING:
            status_str = _("Connecting...");
            button_label = _("Disconnect");
            break;
        default:
            status_str = _("Disconnected");
            button_label = _("Connect");
            break;
        }

        if (NM_IS_CONNECTION_ITEM (item) && nm_connection_item_get_connection (NM_CONNECTION_ITEM (item)))
            str = g_strdup_printf ("<big><b>%s - %s</b></big>", nm_list_item_get_name (item), status_str);
        else
            str = g_strdup_printf ("<big><b>%s</b></big>", nm_list_item_get_name (item));

        gtk_label_set_markup (priv->name_and_status, str);
        g_free (str);

        gtk_button_set_label (GTK_BUTTON (priv->connect_button), button_label);
        update_details (self);
    }

    if (!property || !strcmp (property, NM_LIST_ITEM_ICON))
        gtk_image_set_from_pixbuf (priv->icon, nm_icon_cache_get (nm_list_item_get_icon (item)));

    if (!property || !strcmp (property, NM_LIST_ITEM_SECURITY))
        gtk_label_set_text (priv->security_label, nm_list_item_get_security (item));

    if (!property || !strcmp (property, NM_LIST_ITEM_SHOW_DELETE)) {
        if (nm_list_item_get_show_delete (item))
            gtk_widget_show (priv->remove_button);
        else
            gtk_widget_hide (priv->remove_button);
    }

    if (!property || !strcmp (property, NM_CONNECTION_ITEM_CONNECTION)) {
        update_background (self);

        if (property && NM_IS_CONNECTION_ITEM (item) && 
            !nm_connection_item_get_connection (NM_CONNECTION_ITEM (item))) {
            str = g_strdup_printf ("<big><b>%s</b></big>", nm_list_item_get_name (item));
            gtk_label_set_markup (priv->name_and_status, str);
            g_free (str);
        }
    }
}

static void
connect_button_clicked (GtkButton *button, gpointer user_data)
{
    NmnItemRenderer *self = NMN_ITEM_RENDERER (user_data);
    NMListItem *item;

    item = nmn_item_renderer_get_item (self);
    if (nm_list_item_get_status (item) == NM_LIST_ITEM_STATUS_DISCONNECTED)
        nm_list_item_connect (item);
    else
        nm_list_item_disconnect (item);
}

static void
details_changed (NmnConnectionDetails *details,
                 gboolean complete,
                 gpointer user_data)
{
    NmnItemRendererPrivate *priv = GET_PRIVATE (user_data);

    gtk_widget_set_sensitive (GTK_WIDGET (priv->connect_button), complete);
}

NmnConnectionDetails *
get_details (NmnItemRenderer *self)
{
    NmnItemRendererPrivate *priv = GET_PRIVATE (self);
    GtkWidget *alignment;

    if (priv->details)
        return priv->details;

    if (!NM_IS_DEVICE_ITEM (priv->item) || !nm_device_item_get_device (NM_DEVICE_ITEM (priv->item)))
        return NULL;

    priv->details = nmn_connection_details_new ();
    update_details (self);

    alignment = gtk_alignment_new (0, 0, 0, 0);
    gtk_container_add (GTK_CONTAINER (alignment), GTK_WIDGET (priv->details));
    gtk_widget_show (alignment);
    gtk_box_pack_end (priv->vbox, alignment, FALSE, FALSE, 0);

    g_signal_connect (priv->details, "changed", G_CALLBACK (details_changed), self);
    details_changed (priv->details, nmn_connection_details_verify (priv->details), self);

    return priv->details;
}

static void
advanced_expanded (GtkExpander *expander,
                   GParamSpec *param_spec,
                   gpointer user_data)
{
    NmnConnectionDetails *details;

    details = get_details (NMN_ITEM_RENDERER (user_data));
    if (details)
        g_object_set (details, "visible", gtk_expander_get_expanded (expander), NULL);
}

static void
remove_button_clicked (GtkButton *button, gpointer user_data)
{
    NmnItemRenderer *self = NMN_ITEM_RENDERER (user_data);
    GtkDialog *dialog;
    GtkWidget *label;
    const char *name;
    char *label_text;
    NMListItem *item;

    item = nmn_item_renderer_get_item (self);

    dialog = GTK_DIALOG (gtk_dialog_new_with_buttons (_("Really remove?"),
                                                      NULL,
                                                      GTK_DIALOG_MODAL |
                                                      GTK_DIALOG_DESTROY_WITH_PARENT,
                                                      GTK_STOCK_CANCEL,
                                                      GTK_RESPONSE_REJECT,
                                                      GTK_STOCK_OK,
                                                      GTK_RESPONSE_ACCEPT,
                                                      NULL));

    gtk_dialog_set_has_separator (dialog, FALSE);
    gtk_dialog_set_default_response (dialog, GTK_RESPONSE_ACCEPT);
    gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
    gtk_window_set_icon_name (GTK_WINDOW (dialog), GTK_STOCK_DELETE);

    name = nm_list_item_get_name (item);
    label_text = g_strdup_printf (_("Do you want to remove the '%s' %s network? "
                                    "This\nwill forget the password and you will"
                                    " no longer be\nautomatically connected to "
                                    "'%s'."),
                                  name,
                                  nm_list_item_get_type_name (item),
                                  name);

    label = gtk_label_new (label_text);

    gtk_box_set_spacing (GTK_BOX (dialog->vbox), 12);
    gtk_box_pack_start (GTK_BOX (dialog->vbox), label, TRUE, TRUE, 6);
    gtk_widget_show_all (GTK_WIDGET (dialog));

    if (gtk_dialog_run (dialog) == GTK_RESPONSE_ACCEPT)
        nm_list_item_delete (item);

    gtk_widget_destroy (GTK_WIDGET (dialog));
}

static void
init_widgets (NmnItemRenderer *self)
{
    NmnItemRendererPrivate *priv = GET_PRIVATE (self);
    GtkWidget *vbox;
    GtkWidget *hbox;
    GtkWidget *w;

    priv->vbox = GTK_BOX (gtk_vbox_new (FALSE, 6));
    gtk_container_set_border_width (GTK_CONTAINER (priv->vbox), 6);
    gtk_container_add (GTK_CONTAINER (self), GTK_WIDGET (priv->vbox));

    priv->hbox = GTK_BOX (gtk_hbox_new (FALSE, 6));
    gtk_container_set_border_width (GTK_CONTAINER (priv->hbox), 6);
    gtk_box_pack_start (priv->vbox, GTK_WIDGET (priv->hbox), FALSE, FALSE, 0);

    priv->icon = GTK_IMAGE (gtk_image_new ());
    gtk_box_pack_start (priv->hbox, GTK_WIDGET (priv->icon), FALSE, FALSE, 0);

    vbox = gtk_vbox_new (FALSE, 0);
    gtk_box_pack_start (priv->hbox, vbox, FALSE, FALSE, 0);

    w = gtk_label_new ("");
    gtk_label_set_use_markup (GTK_LABEL (w), TRUE);
    gtk_misc_set_alignment (GTK_MISC (w), 0.0, 0.5);
    gtk_box_pack_start (GTK_BOX (vbox), w, FALSE, FALSE, 0);
    priv->name_and_status = GTK_LABEL (w);

    hbox = gtk_hbox_new (FALSE, 12);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

    w = gtk_label_new ("");
    gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
    priv->security_label = GTK_LABEL (w);

    /* FIXME: this should be visibile only for NMDeviceItems */
    w = gtk_expander_new ("Advanced");
    gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
    priv->expander = w;
    g_signal_connect (w, "notify::expanded", G_CALLBACK (advanced_expanded), self);

    w = gtk_button_new_with_label ("Connect");
    gtk_box_pack_start (GTK_BOX (hbox), w, FALSE, FALSE, 0);
    priv->connect_button = w;
    g_signal_connect (w, "clicked", G_CALLBACK (connect_button_clicked), self);

    /* Remove button */
    vbox = gtk_vbox_new (FALSE, 0);
    gtk_box_pack_end (priv->hbox, vbox, FALSE, FALSE, 0);

    w = gtk_button_new ();
    gtk_button_set_image (GTK_BUTTON (w),
                          gtk_image_new_from_icon_name ("edit-clear", GTK_ICON_SIZE_MENU));

    gtk_button_set_relief (GTK_BUTTON (w), GTK_RELIEF_NONE);
    gtk_button_set_image_position (GTK_BUTTON (w), GTK_POS_RIGHT);
    gtk_widget_set_tooltip_text (w, "Remove connection");
    gtk_box_pack_start (GTK_BOX (vbox), w, TRUE, FALSE, 0);
    priv->remove_button = w;
    g_signal_connect (w, "clicked", G_CALLBACK (remove_button_clicked), self);

    gtk_widget_show_all (GTK_WIDGET (priv->vbox));
}

void
nmn_item_renderer_set_item (NmnItemRenderer *self,
                            NMListItem *item)
{
    NmnItemRendererPrivate *priv;

    g_return_if_fail (NMN_IS_ITEM_RENDERER (self));
    g_return_if_fail (NM_IS_LIST_ITEM (item));

    priv = GET_PRIVATE (self);
    g_return_if_fail (priv->item == NULL);

    init_widgets (self);
    priv->item = g_object_ref (item);
    priv->item_changed_id = g_signal_connect (priv->item, "notify", G_CALLBACK (item_changed), self);
    item_changed (priv->item, NULL, self);
}

static gboolean
enter_notify_event (GtkWidget *widget,
                    GdkEventCrossing *event)
{
    NmnItemRendererPrivate *priv = GET_PRIVATE (widget);

    priv->prelight = TRUE;
    update_background (NMN_ITEM_RENDERER (widget));

    return TRUE;
}

static gboolean
leave_notify_event (GtkWidget *widget,
                    GdkEventCrossing *event)
{
    NmnItemRendererPrivate *priv = GET_PRIVATE (widget);

    if (event->detail != GDK_NOTIFY_INFERIOR) {
        priv->prelight = FALSE;
        update_background (NMN_ITEM_RENDERER (widget));
    }

    return TRUE;
}

static void
nmn_item_renderer_init (NmnItemRenderer *item)
{
    NmnItemRendererPrivate *priv = GET_PRIVATE (item);

    gdk_color_parse ("#cbcbcb", &priv->prelight_color);
    gdk_color_parse ("#e8e8e8", &priv->connection_item_color);
}

static void
dispose (GObject *object)
{
    NmnItemRendererPrivate *priv = GET_PRIVATE (object);

    if (!priv->disposed) {
        if (priv->item) {
            g_signal_handler_disconnect (priv->item, priv->item_changed_id);
            g_object_unref (priv->item);
        }

        priv->disposed = TRUE;
    }

    G_OBJECT_CLASS (nmn_item_renderer_parent_class)->dispose (object);
}

static void
nmn_item_renderer_class_init (NmnItemRendererClass *class)
{
    GObjectClass *object_class = G_OBJECT_CLASS (class);
    GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (class);

    g_type_class_add_private (object_class, sizeof (NmnItemRendererPrivate));

    object_class->dispose = dispose;

    widget_class->enter_notify_event = enter_notify_event;
    widget_class->leave_notify_event = leave_notify_event;
}
